@@ -1,7 +1,8 @@
-from alibabacloud_credentials import providers, credentials
+from functools import wraps
+
+from alibabacloud_credentials import credentials, providers
 from alibabacloud_credentials.models import Config
 from alibabacloud_credentials.utils import auth_constant as ac
-from functools import wraps
 
 
 def attribute_error_return_none(f):
@@ -20,9 +21,9 @@ class Client:
 
     def __init__(self, config=None):
         if config is None:
-            config = Config()
             provider = providers.DefaultCredentialsProvider()
             self.cloud_credential = provider.get_credentials()
+            return
         self.cloud_credential = self.get_credential(config)
 
     @staticmethod
@@ -41,6 +42,8 @@ class Client:
                 0,
                 providers.EcsRamRoleCredentialProvider(config=config)
             )
+        elif config.type == ac.CREDENTIALS_URI:
+            return credentials.CredentialsURICredential(config.credentials_uri)
         elif config.type == ac.RAM_ROLE_ARN:
             return credentials.RamRoleArnCredential(
                 config.access_key_id,
@@ -56,6 +59,13 @@ class Client:
                 0,
                 providers.RsaKeyPairCredentialProvider(config=config)
             )
+        elif config.type == ac.OIDC_ROLE_ARN:
+            return credentials.OIDCRoleArnCredential(
+                config.access_key_id,
+                config.access_key_secret,
+                config.security_token,
+                0,
+                providers.OIDCRoleArnCredentialProvider(config=config))
         return providers.DefaultCredentialsProvider().get_credentials()
 
     def get_access_key_id(self):
