@@ -1,18 +1,19 @@
-import requests
-import json
-import time
-import configparser
 import calendar
+import configparser
+import json
+import os
+import time
 
+import requests
 from Tea.core import TeaCore
 from Tea.request import TeaRequest
 
-from alibabacloud_credentials.utils import auth_util as au, \
-    auth_constant as ac, \
-    parameter_helper as ph
+from alibabacloud_credentials import credentials
 from alibabacloud_credentials.exceptions import CredentialException
 from alibabacloud_credentials.models import Config
-from alibabacloud_credentials import credentials
+from alibabacloud_credentials.utils import auth_constant as ac
+from alibabacloud_credentials.utils import auth_util as au
+from alibabacloud_credentials.utils import parameter_helper as ph
 
 
 class AlibabaCloudCredentialsProvider:
@@ -30,6 +31,8 @@ class AlibabaCloudCredentialsProvider:
             self.role_session_name = config.role_session_name
             self.public_key_id = config.public_key_id
             self.role_name = config.role_name
+            self.oidc_provider_arn = config.oidc_provider_arn
+            self.oidc_token_file_path = config.oidc_token_file_path
             self.private_key_file = config.private_key_file
             self.bearer_token = config.bearer_token
             self.security_token = config.security_token
@@ -66,6 +69,7 @@ class DefaultCredentialsProvider(AlibabaCloudCredentialsProvider):
         role_name = au.environment_ECSMeta_data
         if role_name is not None:
             self.user_configuration_providers.append(EcsRamRoleCredentialProvider(role_name))
+        self.user_configuration_providers.append(CredentialsUriProvider())
 
     def get_credentials(self):
         for provider in self.user_configuration_providers:
@@ -213,8 +217,12 @@ class RamRoleArnCredentialProvider(AlibabaCloudCredentialsProvider):
             'RoleArn': self.role_arn,
             'AccessKeyId': self.access_key_id,
             'RegionId': self.region_id,
-            'RoleSessionName': self.role_session_name
+            'RoleSessionName': self.role_session_name,
+            'SignatureMethod': 'HMAC-SHA1',
+            'SignatureVersion': '1.0'
         }
+        tea_request.query["Timestamp"] = ph.get_iso_8061_date()
+        tea_request.query["SignatureNonce"] = ph.get_uuid()
         if self.policy is not None:
             tea_request.query["Policy"] = self.policy
         string_to_sign = ph.compose_string_to_sign("GET", tea_request.query)
@@ -250,8 +258,12 @@ class RamRoleArnCredentialProvider(AlibabaCloudCredentialsProvider):
             'RoleArn': self.role_arn,
             'AccessKeyId': self.access_key_id,
             'RegionId': self.region_id,
-            'RoleSessionName': self.role_session_name
+            'RoleSessionName': self.role_session_name,
+            'SignatureMethod': 'HMAC-SHA1',
+            'SignatureVersion': '1.0'
         }
+        tea_request.query["Timestamp"] = ph.get_iso_8061_date()
+        tea_request.query["SignatureNonce"] = ph.get_uuid()
         if self.policy is not None:
             tea_request.query["Policy"] = self.policy
         string_to_sign = ph.compose_string_to_sign("GET", tea_request.query)
@@ -274,6 +286,124 @@ class RamRoleArnCredentialProvider(AlibabaCloudCredentialsProvider):
         raise CredentialException(response.body.decode('utf-8'))
 
 
+class OIDCRoleArnCredentialProvider(AlibabaCloudCredentialsProvider):
+    """OIDCRoleArnCredentialProvider"""
+
+    def __init__(self, access_key_id=None, access_key_secret=None, role_session_name=None, role_arn=None,
+                 oidc_provider_arn=None,
+                 oidc_token_file_path=None,
+                 region_id=None,
+                 policy=None, config=None):
+        self._verify_empty_args(access_key_id, access_key_secret, config=config)
+        super().__init__(config)
+        self._set_arg('role_arn', role_arn)
+        self._set_arg('oidc_provider_arn', oidc_provider_arn)
+        if oidc_token_file_path is not None:
+            self._set_arg('oidc_token_file_path', oidc_token_file_path)
+        elif config.oidc_token_file_path is not None:
+            self._set_arg('oidc_token_file_path', oidc_token_file_path)
+        elif au.environment_oidc_token_file is not None:
+            self._set_arg('oidc_token_file_path', au.environment_oidc_token_file)
+        else:
+            raise CredentialException(
+                'The oidc_token_file_path does not exist and env ALIBABA_CLOUD_OIDC_TOKEN_FILE is none.')
+        self._set_arg('access_key_id', access_key_id)
+        self._set_arg('access_key_secret', access_key_secret)
+        self._set_arg('region_id', region_id)
+        self._set_arg('role_session_name', role_session_name)
+        self._set_arg('policy', policy)
+
+    def get_credentials(self):
+        return self._create_credentials()
+
+    def _create_credentials(self, turl=None):
+        # 获取credential 先实现签名用工具类
+        oidc_token = au.get_private_key(self.oidc_token_file_path)
+        tea_request = TeaRequest()
+        tea_request.query = {
+            'Action': 'AssumeRoleWithOIDC',
+            'Format': 'JSON',
+            'Version': '2015-04-01',
+            'RegionId': self.region_id,
+            'DurationSeconds': str(self.duration_seconds),
+            'RoleArn': self.role_arn,
+            'OIDCProviderArn': self.oidc_provider_arn,
+            'OIDCToken': oidc_token,
+            'RoleSessionName': self.role_session_name,
+            'SignatureMethod': 'HMAC-SHA1',
+            'SignatureVersion': '1.0'
+        }
+        tea_request.query["Timestamp"] = ph.get_iso_8061_date()
+        tea_request.query["SignatureNonce"] = ph.get_uuid()
+        if self.policy is not None:
+            tea_request.query["Policy"] = self.policy
+        string_to_sign = ph.compose_string_to_sign("GET", tea_request.query)
+        if self.access_key_id is not None and self.access_key_secret is not None:
+            tea_request.query["AccessKeyId"] = self.access_key_id
+            signature = ph.sign_string(string_to_sign, self.access_key_secret + "&")
+            tea_request.query["Signature"] = signature
+        tea_request.protocol = 'https'
+        tea_request.headers['host'] = turl if turl else 'sts.aliyuncs.com'
+        # request
+        response = TeaCore.do_action(tea_request)
+        if response.status_code == 200:
+            dic = json.loads(response.body.decode('utf-8'))
+            if "Credentials" in dic:
+                cre = dic.get("Credentials")
+                # 先转换为时间数组
+                time_array = time.strptime(cre.get("Expiration"), "%Y-%m-%dT%H:%M:%SZ")
+                # 转换为时间戳
+                expiration = calendar.timegm(time_array)
+                return credentials.OIDCRoleArnCredential(cre.get("AccessKeyId"), cre.get("AccessKeySecret"),
+                                                         cre.get("SecurityToken"), expiration, self)
+        raise CredentialException(response.body.decode('utf-8'))
+
+    async def get_credentials_async(self):
+        return await self._create_credentials_async()
+
+    async def _create_credentials_async(self, turl=None):
+        # 获取credential 先实现签名用工具类
+        oidc_token = au.get_private_key(self.oidc_token_file_path)
+        tea_request = TeaRequest()
+        tea_request.query = {
+            'Action': 'AssumeRoleWithOIDC',
+            'Format': 'JSON',
+            'Version': '2015-04-01',
+            'RegionId': self.region_id,
+            'DurationSeconds': str(self.duration_seconds),
+            'RoleArn': self.role_arn,
+            'OIDCProviderArn': self.oidc_provider_arn,
+            'OIDCToken': oidc_token,
+            'RoleSessionName': self.role_session_name,
+            'SignatureMethod': 'HMAC-SHA1',
+            'SignatureVersion': '1.0'
+        }
+        tea_request.query["Timestamp"] = ph.get_iso_8061_date()
+        tea_request.query["SignatureNonce"] = ph.get_uuid()
+        if self.policy is not None:
+            tea_request.query["Policy"] = self.policy
+        string_to_sign = ph.compose_string_to_sign("GET", tea_request.query)
+        if self.access_key_id is not None and self.access_key_secret is not None:
+            tea_request.query["AccessKeyId"] = self.access_key_id
+            signature = ph.sign_string(string_to_sign, self.access_key_secret + "&")
+            tea_request.query["Signature"] = signature
+        tea_request.protocol = 'https'
+        tea_request.headers['host'] = turl if turl else 'sts.aliyuncs.com'
+        # request
+        response = TeaCore.async_do_action(tea_request)
+        if response.status_code == 200:
+            dic = json.loads(response.body.decode('utf-8'))
+            if "Credentials" in dic:
+                cre = dic.get("Credentials")
+                # 先转换为时间数组
+                time_array = time.strptime(cre.get("Expiration"), "%Y-%m-%dT%H:%M:%SZ")
+                # 转换为时间戳
+                expiration = calendar.timegm(time_array)
+                return credentials.OIDCRoleArnCredential(cre.get("AccessKeyId"), cre.get("AccessKeySecret"),
+                                                         cre.get("SecurityToken"), expiration, self)
+        raise CredentialException(response.body.decode('utf-8'))
+
+
 class RsaKeyPairCredentialProvider(AlibabaCloudCredentialsProvider):
 
     def __init__(self, access_key_id=None, access_key_secret=None, region_id=None, config=None):
@@ -295,7 +425,11 @@ class RsaKeyPairCredentialProvider(AlibabaCloudCredentialsProvider):
             'DurationSeconds': str(self.duration_seconds),
             'AccessKeyId': self.access_key_id,
             'RegionId': self.region_id,
+            'SignatureMethod': 'HMAC-SHA1',
+            'SignatureVersion': '1.0'
         }
+        tea_request.query["Timestamp"] = ph.get_iso_8061_date()
+        tea_request.query["SignatureNonce"] = ph.get_uuid()
 
         str_to_sign = ph.compose_string_to_sign('GET', tea_request.query)
         signature = ph.sign_string(str_to_sign, self.access_key_id + '&')
@@ -310,7 +444,8 @@ class RsaKeyPairCredentialProvider(AlibabaCloudCredentialsProvider):
                 cre = dic.get("SessionAccessKey")
                 time_array = time.strptime(cre.get("Expiration"), "%Y-%m-%dT%H:%M:%SZ")
                 expiration = calendar.timegm(time_array)
-                return credentials.RsaKeyPairCredential(cre.get("SessionAccessKeyId"), cre.get("SessionAccessKeySecret"),
+                return credentials.RsaKeyPairCredential(cre.get("SessionAccessKeyId"),
+                                                        cre.get("SessionAccessKeySecret"),
                                                         expiration, self)
         raise CredentialException(response.body.decode('utf-8'))
 
@@ -326,7 +461,11 @@ class RsaKeyPairCredentialProvider(AlibabaCloudCredentialsProvider):
             'DurationSeconds': str(self.duration_seconds),
             'AccessKeyId': self.access_key_id,
             'RegionId': self.region_id,
+            'SignatureMethod': 'HMAC-SHA1',
+            'SignatureVersion': '1.0'
         }
+        tea_request.query["Timestamp"] = ph.get_iso_8061_date()
+        tea_request.query["SignatureNonce"] = ph.get_uuid()
 
         str_to_sign = ph.compose_string_to_sign('GET', tea_request.query)
         signature = ph.sign_string(str_to_sign, self.access_key_id + '&')
@@ -341,7 +480,8 @@ class RsaKeyPairCredentialProvider(AlibabaCloudCredentialsProvider):
                 cre = dic.get("SessionAccessKey")
                 time_array = time.strptime(cre.get("Expiration"), "%Y-%m-%dT%H:%M:%SZ")
                 expiration = calendar.timegm(time_array)
-                return credentials.RsaKeyPairCredential(cre.get("SessionAccessKeyId"), cre.get("SessionAccessKeySecret"),
+                return credentials.RsaKeyPairCredential(cre.get("SessionAccessKeyId"),
+                                                        cre.get("SessionAccessKeySecret"),
                                                         expiration, self)
         raise CredentialException(response.body.decode('utf-8'))
 
@@ -354,7 +494,9 @@ class ProfileCredentialsProvider(AlibabaCloudCredentialsProvider):
     def parse_ini(self):
         file_path = self.file_path if self.file_path else au.environment_credentials_file
         if file_path is None:
-            file_path = ac.DEFAULT_CREDENTIALS_FILE_PATH
+            if not ac.HOME:
+                return
+            file_path = os.path.join(ac.HOME, "/.alibabacloud/credentials.ini")
         if len(file_path) == 0:
             raise CredentialException("The specified credentials file is empty")
 
@@ -385,6 +527,8 @@ class ProfileCredentialsProvider(AlibabaCloudCredentialsProvider):
             raise CredentialException("The configured client type is empty")
         elif ac.INI_TYPE_ARN == config_type:
             return self._get_sts_assume_role_session_provider(config).get_credentials()
+        elif ac.INI_TYPE_OIDC == config_type:
+            return self._get_sts_oidc_role_session_provider(config).get_credentials()
         elif ac.INI_TYPE_KEY_PAIR == config_type:
             return self._get_sts_get_session_access_key_provider(config).get_credentials()
         elif ac.INI_TYPE_RAM == config_type:
@@ -413,6 +557,26 @@ class ProfileCredentialsProvider(AlibabaCloudCredentialsProvider):
             access_key_id, access_key_secret, role_session_name, role_arn, region_id, policy
         )
 
+    @staticmethod
+    def _get_sts_oidc_role_session_provider(config):
+        access_key_id = config.get(ac.INI_ACCESS_KEY_ID)
+        access_key_secret = config.get(ac.INI_ACCESS_KEY_IDSECRET)
+        role_session_name = config.get(ac.INI_ROLE_SESSION_NAME)
+        role_arn = config.get(ac.INI_ROLE_ARN)
+        oidc_provider_arn = config.get(ac.INI_OIDC_PROVIDER_ARN)
+        oidc_token_file_path = config.get(ac.INI_OIDC_TOKEN_FILE_PATH)
+        region_id = config.get(ac.DEFAULT_REGION)
+        policy = config.get(ac.INI_POLICY)
+
+        if not role_arn:
+            raise CredentialException("The configured role_arn is empty")
+        if not oidc_provider_arn:
+            raise CredentialException("The configured oidc_provider_arn is empty")
+        return OIDCRoleArnCredentialProvider(
+            access_key_id, access_key_secret, role_session_name, role_arn, oidc_provider_arn, oidc_token_file_path,
+            region_id, policy
+        )
+
     @staticmethod
     def _get_sts_get_session_access_key_provider(config):
         public_key_id = config.get(ac.INI_PUBLIC_KEY_ID)
@@ -446,3 +610,11 @@ class EnvironmentVariableCredentialsProvider(AlibabaCloudCredentialsProvider):
         if len(access_key_secret) == 0:
             raise CredentialException("Environment variable accessKeySecret cannot be empty")
         return credentials.AccessKeyCredential(access_key_id, access_key_secret)
+
+
+class CredentialsUriProvider(AlibabaCloudCredentialsProvider):
+    def get_credentials(self):
+        credentials_uri = os.environ.get('ALIBABA_CLOUD_CREDENTIALS_URI')
+        if credentials_uri is None:
+            return None
+        return credentials.CredentialsURICredential(credentials_uri)
